@@ -60,6 +60,7 @@ class FAISSDocumentStore(SQLDocumentStore):
         n_links: int = 64,
         ef_search: int = 20,
         ef_construction: int = 80,
+        validate_index_sync: bool = True,
     ):
         """
         :param sql_url: SQL connection URL for database. It defaults to local file based SQLite DB. For large scale
@@ -107,6 +108,7 @@ class FAISSDocumentStore(SQLDocumentStore):
         :param n_links: used only if index_factory == "HNSW"
         :param ef_search: used only if index_factory == "HNSW"
         :param ef_construction: used only if index_factory == "HNSW"
+        :param validate_index_sync: Whether to check that the document count equals the embedding count at initialization time
         """
         # special case if we want to load an existing index from disk
         # load init params from disk and run init again
@@ -162,7 +164,8 @@ class FAISSDocumentStore(SQLDocumentStore):
             url=sql_url, index=index, duplicate_documents=duplicate_documents, isolation_level=isolation_level
         )
 
-        self._validate_index_sync()
+        if validate_index_sync:
+            self._validate_index_sync()
 
     def _validate_params_load_from_disk(self, sig: Signature, locals: dict):
         allowed_params = ["faiss_index_path", "faiss_config_path", "self"]
