@@ -52,6 +52,8 @@ class PineconeDocumentStore(SQLDocumentStore):
         progress_bar: bool = True,
         duplicate_documents: str = "overwrite",
         recreate_index: bool = False,
+        metadata_config: dict = {"indexed": []},
+        validate_index_sync: bool = True,
     ):
         """
         :param api_key: Pinecone vector database API key ([https://app.pinecone.io](https://app.pinecone.io)).
@@ -85,6 +87,10 @@ class PineconeDocumentStore(SQLDocumentStore):
             created using the config you are using for initialization. Be aware that all data in the old index will be
             lost if you choose to recreate the index. Be aware that both the document_index and the label_index will
             be recreated.
+        :param metadata_config: Which metadata fields should be indexed. Should be in the format
+            `{"indexed": ["metadata-field-1", "metadata-field-2", "metadata-field-n"]}`.
+            Indexing metadata fields is a prerequisite to allow filtering of documents by metadata values.
+        :param validate_index_sync: Whether to check that the document count equals the embedding count at initialization time
         """
         # Connect to Pinecone server using python client binding
         pinecone.init(api_key=api_key, environment=environment)
@@ -113,6 +119,7 @@ class PineconeDocumentStore(SQLDocumentStore):
         # Pinecone index params
         self.replicas = replicas
         self.shards = shards
+        self.metadata_config = metadata_config
 
         # Initialize dictionary of index connections
         self.pinecone_indexes: Dict[str, pinecone.Index] = {}
@@ -134,8 +141,12 @@ class PineconeDocumentStore(SQLDocumentStore):
                 replicas=self.replicas,
                 shards=self.shards,
                 recreate_index=recreate_index,
+                metadata_config=self.metadata_config,
             )
 
+        if validate_index_sync:
+            self._validate_index_sync()
+
     def _sanitize_index_name(self, index: str) -> str:
         return index.replace("_", "-").lower()
 
@@ -147,6 +158,7 @@ class PineconeDocumentStore(SQLDocumentStore):
         replicas: Optional[int] = 1,
         shards: Optional[int] = 1,
         recreate_index: bool = False,
+        metadata_config: dict = {},
     ):
         """
         Create a new index for storing documents in case an
@@ -166,7 +178,12 @@ class PineconeDocumentStore(SQLDocumentStore):
             # Search pinecone hosted indexes and create an index if it does not exist
             if index not in pinecone.list_indexes():
                 pinecone.create_index(
-                    name=index, dimension=embedding_dim, metric=metric_type, replicas=replicas, shards=shards
+                    name=index,
+                    dimension=embedding_dim,
+                    metric=metric_type,
+                    replicas=replicas,
+                    shards=shards,
+                    metadata_config=metadata_config,
                 )
             index_connection = pinecone.Index(index)
 
@@ -256,7 +273,7 @@ class PineconeDocumentStore(SQLDocumentStore):
                         if self.similarity == "cosine":
                             self.normalize_embedding(embeddings_to_index)
                         # Convert embeddings to list objects
-                        embeddings = [embed.tolist() for embed in embeddings]
+                        embeddings = [embed.tolist() if embed is not None else None for embed in embeddings]
                         data_to_write_to_pinecone = zip(ids, embeddings, metadata)
                         # Metadata fields and embeddings are stored in Pinecone
                         self.pinecone_indexes[index].upsert(vectors=data_to_write_to_pinecone)
@@ -670,8 +687,8 @@ class PineconeDocumentStore(SQLDocumentStore):
                 f"Index named '{index}' does not exist. Try reinitializing PineconeDocumentStore() and running "
                 f"'update_embeddings()' to create and populate an index."
             )
+        query_emb = query_emb.astype(np.float32)
 
-        query_emb = query_emb.reshape(1, -1).astype(np.float32)
         if self.similarity == "cosine":
             self.normalize_embedding(query_emb)
 
@@ -679,7 +696,7 @@ class PineconeDocumentStore(SQLDocumentStore):
 
         score_matrix = []
         vector_id_matrix = []
-        for match in res["results"][0]["matches"]:
+        for match in res["matches"]:
             score_matrix.append(match["score"])
             vector_id_matrix.append(match["id"])
         documents = self.get_documents_by_id(vector_id_matrix, index=index, return_embedding=return_embedding)
