@@ -130,6 +130,9 @@ class EvalDocuments(BaseComponent):
             )
         return {"correct_retrieval": correct_retrieval}, "output_1"
 
+    def run_batch(self):  # type: ignore
+        raise NotImplementedError("run_batch not supported for EvalDocuments node.")
+
     def reciprocal_rank_retrieved(self, retriever_labels, predictions, top_k_eval_documents):
         if self.open_domain:
             for answer in retriever_labels.answers:
@@ -266,7 +269,7 @@ class EvalAnswers(BaseComponent):
                 # Compute Semantic Answer Similarity if model is supplied
                 if self.sas_model is not None:
                     # sas works on batches, so we pack the labels into a list of lists, and unpack the return values as well
-                    top_1_sas, top_k_sas = semantic_answer_similarity(
+                    top_1_sas, top_k_sas, _ = semantic_answer_similarity(
                         predictions=[predictions_str],
                         gold_labels=[multi_labels.answers],
                         sas_model_name_or_path=self.sas_model,
@@ -293,12 +296,15 @@ class EvalAnswers(BaseComponent):
                 self.update_has_answer_metrics()
         return {}, "output_1"
 
+    def run_batch(self):  # type: ignore
+        raise NotImplementedError("run_batch not supported for EvalAnswers node.")
+
     def evaluate_extraction(self, gold_labels: List[str], predictions: List[str]):
         if self.open_domain:
             top_1_em = calculate_em_str_multi(gold_labels, predictions[0])
             top_1_f1 = calculate_f1_str_multi(gold_labels, predictions[0])
-            top_k_em = max([calculate_em_str_multi(gold_labels, p) for p in predictions])
-            top_k_f1 = max([calculate_f1_str_multi(gold_labels, p) for p in predictions])
+            top_k_em = max(calculate_em_str_multi(gold_labels, p) for p in predictions)
+            top_k_f1 = max(calculate_f1_str_multi(gold_labels, p) for p in predictions)
         else:
             logger.error(
                 "Closed Domain Reader Evaluation not yet implemented for Pipelines. Use Reader.eval() instead."
@@ -395,11 +401,12 @@ def semantic_answer_similarity(
     sas_model_name_or_path: str = "sentence-transformers/paraphrase-multilingual-mpnet-base-v2",
     batch_size: int = 32,
     use_gpu: bool = True,
-) -> Tuple[List[float], List[float]]:
+) -> Tuple[List[float], List[float], List[List[float]]]:
     """
     Computes Transformer-based similarity of predicted answer to gold labels to derive a more meaningful metric than EM or F1.
     Returns per QA pair a) the similarity of the most likely prediction (top 1) to all available gold labels
                         b) the highest similarity of all predictions to gold labels
+                        c) a matrix consisting of the similarities of all the predicitions compared to all gold labels
 
     :param predictions: Predicted answers as list of multiple preds per question
     :param gold_labels: Labels as list of multiple possible answers per question
@@ -408,7 +415,7 @@ def semantic_answer_similarity(
     :param batch_size: Number of prediction label pairs to encode at once.
     :param use_gpu: Whether to use a GPU or the CPU for calculating semantic answer similarity.
                     Falls back to CPU if no GPU is available.
-    :return: top_1_sas, top_k_sas
+    :return: top_1_sas, top_k_sas, pred_label_matrix
     """
     assert len(predictions) == len(gold_labels)
 
@@ -422,6 +429,7 @@ def semantic_answer_similarity(
     # Compute similarities
     top_1_sas = []
     top_k_sas = []
+    pred_label_matrix = []
     lengths: List[Tuple[int, int]] = []
 
     # Based on Modelstring we can load either Bi-Encoders or Cross Encoders.
@@ -443,6 +451,7 @@ def semantic_answer_similarity(
             # So to only consider the first doc we have to take the first len_l entries
             top_1_sas.append(np.max(scores_window[:len_l]))
             top_k_sas.append(np.max(scores_window))
+            pred_label_matrix.append(scores_window.reshape(len_p, len_l).tolist())
             current_position += len_p * len_l
     else:
         # For Bi-encoders we can flatten predictions and labels into one list
@@ -466,8 +475,9 @@ def semantic_answer_similarity(
             sims = cosine_similarity(pred_embeddings, label_embeddings)
             top_1_sas.append(np.max(sims[0, :]))
             top_k_sas.append(np.max(sims))
+            pred_label_matrix.append(sims.tolist())
 
-    return top_1_sas, top_k_sas
+    return top_1_sas, top_k_sas, pred_label_matrix
 
 
 def _count_overlap(
