@@ -5,10 +5,12 @@ from functools import partial, reduce
 from itertools import chain
 from typing import List, Optional, Generator, Set, Union
 import warnings
+from pathlib import Path
+from pickle import UnpicklingError
 
 import nltk
 from more_itertools import windowed
-from tqdm import tqdm
+from tqdm.auto import tqdm
 
 from haystack.nodes.preprocessor.base import BasePreProcessor
 from haystack.errors import HaystackError
@@ -51,8 +53,11 @@ class PreProcessor(BasePreProcessor):
         split_length: int = 200,
         split_overlap: int = 0,
         split_respect_sentence_boundary: bool = True,
+        tokenizer_model_folder: Optional[Union[str, Path]] = None,
         language: str = "en",
         id_hash_keys: Optional[List[str]] = None,
+        progress_bar: bool = True,
+        add_page_number: bool = False,
     ):
         """
         :param clean_header_footer: Use heuristic to remove footers and headers across different pages by searching
@@ -75,10 +80,16 @@ class PreProcessor(BasePreProcessor):
                                                 to True, the individual split will always have complete sentences &
                                                 the number of words will be <= split_length.
         :param language: The language used by "nltk.tokenize.sent_tokenize" in iso639 format. Available options: "en", "es", "de", "fr" & many more.
+        :param tokenizer_model_folder: Path to the folder containing the NTLK PunktSentenceTokenizer models, if loading a model from a local path. Leave empty otherwise.
         :param id_hash_keys: Generate the document id from a custom list of strings that refer to the document's
             attributes. If you want to ensure you don't have duplicate documents in your DocumentStore but texts are
             not unique, you can modify the metadata and pass e.g. `"meta"` to this field (e.g. [`"content"`, `"meta"`]).
             In this case the id will be generated by using the content and the defined metadata.
+        :param progress_bar: Whether to show a progress bar.
+        :param add_page_number: Add the number of the page a paragraph occurs in to the Document's meta
+                                field `"page"`. Page boundaries are determined by `"\f"' character which is added
+                                in between pages by `PDFToTextConverter`, `TikaConverter`, `ParsrConverter` and
+                                `AzureConverter`.
         """
         super().__init__()
 
@@ -95,9 +106,12 @@ class PreProcessor(BasePreProcessor):
         self.split_length = split_length
         self.split_overlap = split_overlap
         self.split_respect_sentence_boundary = split_respect_sentence_boundary
-        self.language = iso639_to_nltk.get(language, language)
+        self.language = language
+        self.tokenizer_model_folder = tokenizer_model_folder
         self.print_log: Set[str] = set()
         self.id_hash_keys = id_hash_keys
+        self.progress_bar = progress_bar
+        self.add_page_number = add_page_number
 
     def process(
         self,
@@ -203,7 +217,8 @@ class PreProcessor(BasePreProcessor):
         self, documents: List[Union[dict, Document]], id_hash_keys: Optional[List[str]] = None, **kwargs
     ) -> List[Document]:
         nested_docs = [
-            self._process_single(d, id_hash_keys=id_hash_keys, **kwargs) for d in tqdm(documents, unit="docs")
+            self._process_single(d, id_hash_keys=id_hash_keys, **kwargs)
+            for d in tqdm(documents, disable=not self.progress_bar, desc="Preprocessing", unit="docs")
         ]
         return [d for x in nested_docs for d in x]
 
@@ -229,6 +244,11 @@ class PreProcessor(BasePreProcessor):
         # Mainly needed for type checking
         if not isinstance(document, Document):
             raise HaystackError("Document must not be of type 'dict' but of type 'Document'.")
+
+        if type(document.content) is not str:
+            logger.error("Document content is not of type str. Nothing to clean.")
+            return document
+
         text = document.content
         if clean_header_footer:
             text = self._find_and_remove_header_footer(
@@ -236,13 +256,20 @@ class PreProcessor(BasePreProcessor):
             )
 
         if clean_whitespace:
-            lines = text.splitlines()
+            pages = text.split("\f")
+            cleaned_pages = []
+            for page in pages:
+                if not page:
+                    continue
+                lines = page.splitlines()
+                cleaned_lines = []
+                for line in lines:
+                    line = line.strip()
+                    cleaned_lines.append(line)
+                cleaned_page = "\n".join(cleaned_lines)
+                cleaned_pages.append(cleaned_page)
 
-            cleaned_lines = []
-            for line in lines:
-                line = line.strip()
-                cleaned_lines.append(line)
-            text = "\n".join(cleaned_lines)
+            text = "\f".join(cleaned_pages)
 
         if clean_empty_lines:
             text = re.sub(r"\n\n+", "\n\n", text)
@@ -286,43 +313,78 @@ class PreProcessor(BasePreProcessor):
         if split_respect_sentence_boundary and split_by != "word":
             raise NotImplementedError("'split_respect_sentence_boundary=True' is only compatible with split_by='word'.")
 
+        if type(document.content) is not str:
+            logger.error("Document content is not of type str. Nothing to split.")
+            return [document]
+
         text = document.content
 
         if split_respect_sentence_boundary and split_by == "word":
             # split by words ensuring no sub sentence splits
-            sentences = nltk.tokenize.sent_tokenize(text, language=self.language)
-            word_count = 0
+            if self.add_page_number:
+                # SentenceTokenizer will remove "\f" if it is at the end of a sentence, so substituting it in these
+                # cases for "[NEW_PAGE]" to don't lose any page breaks.
+                text = self._substitute_page_breaks(text)
+            sentences = self._split_sentences(text)
+
+            word_count_slice = 0
+            cur_page = 1
+            splits_pages = []
             list_splits = []
             current_slice: List[str] = []
             for sen in sentences:
-                current_word_count = len(sen.split(" "))
-                if current_word_count > split_length:
+                if self.add_page_number and sen.startswith("[NEW_PAGE]"):
+                    sen = sen.replace("[NEW_PAGE]", "\f")
+
+                word_count_sen = len(sen.split(" "))
+                if word_count_sen > split_length:
                     long_sentence_message = f"One or more sentence found with word count higher than the split length."
                     if long_sentence_message not in self.print_log:
                         self.print_log.add(long_sentence_message)
                         logger.warning(long_sentence_message)
-                if word_count + current_word_count > split_length:
-                    list_splits.append(current_slice)
-                    # Enable split_stride with split_by='word' while respecting sentence boundaries.
+                if word_count_slice + word_count_sen > split_length:
+                    # Number of words exceeds split_length -> save current slice and start a new one
+                    if current_slice:
+                        list_splits.append(current_slice)
+                        splits_pages.append(cur_page)
+
                     if split_overlap:
                         overlap = []
-                        w_count = 0
-                        for s in current_slice[::-1]:
+                        processed_sents = []
+                        word_count_overlap = 0
+                        current_slice_copy = deepcopy(current_slice)
+                        for idx, s in reversed(list(enumerate(current_slice))):
                             sen_len = len(s.split(" "))
-                            if w_count < split_overlap:
+                            if word_count_overlap < split_overlap:
                                 overlap.append(s)
-                                w_count += sen_len
+                                word_count_overlap += sen_len
+                                current_slice_copy.pop(idx)
                             else:
+                                processed_sents = current_slice_copy
                                 break
                         current_slice = list(reversed(overlap))
-                        word_count = w_count
+                        word_count_slice = word_count_overlap
                     else:
+                        processed_sents = current_slice
                         current_slice = []
-                        word_count = 0
+                        word_count_slice = 0
+
+                    # Count number of page breaks in processed sentences
+                    if self.add_page_number:
+                        num_page_breaks = self._count_processed_page_breaks(
+                            sentences=processed_sents,
+                            split_overlap=split_overlap,
+                            overlapping_sents=current_slice,
+                            current_sent=sen,
+                        )
+                        cur_page += num_page_breaks
+
                 current_slice.append(sen)
-                word_count += len(sen.split(" "))
+                word_count_slice += word_count_sen
+
             if current_slice:
                 list_splits.append(current_slice)
+                splits_pages.append(cur_page)
 
             text_splits = []
             for sl in list_splits:
@@ -334,7 +396,11 @@ class PreProcessor(BasePreProcessor):
             if split_by == "passage":
                 elements = text.split("\n\n")
             elif split_by == "sentence":
-                elements = nltk.tokenize.sent_tokenize(text, language=self.language)
+                if self.add_page_number:
+                    # SentenceTokenizer will remove "\f" if it is at the end of a sentence, so substituting it in these
+                    # cases for "[NEW_PAGE]" to don't lose any page breaks.
+                    text = self._substitute_page_breaks(text)
+                elements = self._split_sentences(text)
             elif split_by == "word":
                 elements = text.split(" ")
             else:
@@ -348,16 +414,26 @@ class PreProcessor(BasePreProcessor):
             else:
                 segments = windowed(elements, n=split_length, step=split_length)
             text_splits = []
+            splits_pages = []
+            cur_page = 1
             for seg in segments:
-                txt = " ".join([t for t in seg if t is not None])
+                current_units = [unit for unit in seg if unit is not None]
+                txt = " ".join(current_units)
                 if len(txt) > 0:
                     text_splits.append(txt)
+                    splits_pages.append(cur_page)
+                    if self.add_page_number:
+                        processed_units = current_units[: split_length - split_overlap]
+                        num_page_breaks = sum(processed_unit.count("\f") for processed_unit in processed_units)
+                        cur_page += num_page_breaks
 
         # create new document dicts for each text split
         documents = []
         for i, txt in enumerate(text_splits):
             doc = Document(content=txt, meta=deepcopy(document.meta) or {}, id_hash_keys=id_hash_keys)
             doc.meta["_split_id"] = i
+            if self.add_page_number:
+                doc.meta["page"] = splits_pages[i]
             documents.append(doc)
 
         return documents
@@ -444,3 +520,81 @@ class PreProcessor(BasePreProcessor):
             # no common sequence found
             longest = ""
         return longest if longest.strip() else None
+
+    def _split_sentences(self, text: str) -> List[str]:
+        """
+        Tokenize text into sentences.
+        :param text: str, text to tokenize
+        :return: list[str], list of sentences
+        """
+        sentences = []
+
+        language_name = iso639_to_nltk.get(self.language)
+
+        # Try to load a custom model from 'tokenizer_model_path'
+        if self.tokenizer_model_folder is not None:
+            tokenizer_model_path = Path(self.tokenizer_model_folder).absolute() / f"{self.language}.pickle"
+            try:
+                sentence_tokenizer = nltk.data.load(f"file:{str(tokenizer_model_path)}", format="pickle")
+                sentences = sentence_tokenizer.tokenize(text)
+            except LookupError:
+                logger.exception(f"PreProcessor couldn't load sentence tokenizer from {str(tokenizer_model_path)}")
+            except (UnpicklingError, ValueError) as e:
+                logger.exception(
+                    f"PreProcessor couldn't determine model format of sentence tokenizer at {str(tokenizer_model_path)}."
+                )
+            if sentences:
+                return sentences
+
+            # NLTK failed to split, fallback to the default model or to English
+            if language_name is not None:
+                logger.error(
+                    f"PreProcessor couldn't find custom sentence tokenizer model for {self.language}. Using default {self.language} model."
+                )
+                return nltk.tokenize.sent_tokenize(text, language=language_name)
+
+            logger.error(
+                f"PreProcessor couldn't find default or custom sentence tokenizer model for {self.language}. Using English instead."
+            )
+            return nltk.tokenize.sent_tokenize(text, language="english")
+
+        # Use a default NLTK model
+        if language_name is not None:
+            return nltk.tokenize.sent_tokenize(text, language=language_name)
+
+        logger.error(
+            f"PreProcessor couldn't find default sentence tokenizer model for {self.language}. Using English instead. "
+            "You may train your own model and use the 'tokenizer_model_folder' parameter."
+        )
+        return nltk.tokenize.sent_tokenize(text, language="english")
+
+    @staticmethod
+    def _count_processed_page_breaks(
+        sentences: List[str], split_overlap: int, overlapping_sents: List[str], current_sent: str
+    ) -> int:
+        """
+        Counts the number of processed page breaks in a list of processed sentences.
+        """
+        num_page_breaks = sum(sent.count("\f") for sent in sentences)
+        if sentences and sentences[0].startswith("\f"):
+            # Remove already used page break
+            num_page_breaks -= 1
+        # Increment page counter if new split starts with a page break
+        if split_overlap and overlapping_sents:
+            if overlapping_sents[0].startswith("\f"):
+                num_page_breaks += 1
+        else:
+            if current_sent.startswith("\f"):
+                num_page_breaks += 1
+
+        return num_page_breaks
+
+    @staticmethod
+    def _substitute_page_breaks(text: str) -> str:
+        """
+        This method substitutes the page break character "\f" for "[NEW_PAGE]" if it is at the end of a sentence.
+        """
+        # This regex matches any of sentence-ending punctuation (one of ".", ":", "?", "!") followed by a page break
+        # character ("\f") and replaces the page break character with "[NEW_PAGE]" keeping the original sentence-ending
+        # punctuation.
+        return re.sub(r"([\.:?!])\f", r"\1 [NEW_PAGE]", text)
