@@ -4,8 +4,12 @@ import logging
 import multiprocessing
 from pathlib import Path
 from collections import defaultdict
+import os
+import tempfile
 from time import perf_counter
+
 import torch
+from huggingface_hub import create_repo, HfFolder, Repository
 
 from haystack.errors import HaystackError
 from haystack.modeling.data_handler.data_silo import DataSilo, DistillationDataSilo
@@ -104,12 +108,14 @@ class FARMReader(BaseReader):
                              Can be helpful to disable in production deployments to keep the logs clean.
         :param duplicate_filtering: Answers are filtered based on their position. Both start and end position of the answers are considered.
                                     The higher the value, answers that are more apart are filtered out. 0 corresponds to exact duplicates. -1 turns off duplicate removal.
-        :param use_confidence_scores: Sets the type of score that is returned with every predicted answer.
+        :param use_confidence_scores: Determines the type of score that is used for ranking a predicted answer.
                                       `True` => a scaled confidence / relevance score between [0, 1].
                                       This score can also be further calibrated on your dataset via self.eval()
-                                      (see https://haystack.deepset.ai/components/reader#confidence-scores) .
+                                      (see https://haystack.deepset.ai/components/reader#confidence-scores).
                                       `False` => an unscaled, raw score [-inf, +inf] which is the sum of start and end logit
                                       from the model for the predicted span.
+                                      Using confidence scores can change the ranking of no_answer compared to using the
+                                      unscaled raw scores.
         :param confidence_threshold: Filters out predictions below confidence_threshold. Value should be between 0 and 1. Disabled by default.
         :param proxies: Dict of proxy servers to use for downloading external models. Example: {'http': 'some.proxy:1234', 'http://hostname': 'my.proxy:3111'}
         :param local_files_only: Whether to force checking for local files only (and forbid downloads)
@@ -183,6 +189,7 @@ class FARMReader(BaseReader):
         temperature: float = 1.0,
         tinybert: bool = False,
         processor: Optional[Processor] = None,
+        grad_acc_steps: int = 1,
     ):
         if dev_filename:
             dev_split = 0
@@ -259,6 +266,7 @@ class FARMReader(BaseReader):
             n_epochs=n_epochs,
             device=devices[0],
             use_amp=use_amp,
+            grad_acc_steps=grad_acc_steps,
         )
         # 4. Feed everything to the Trainer, which keeps care of growing our model and evaluates it from time to time
         if tinybert:
@@ -279,6 +287,7 @@ class FARMReader(BaseReader):
                 checkpoint_root_dir=Path(checkpoint_root_dir),
                 checkpoint_every=checkpoint_every,
                 checkpoints_to_keep=checkpoints_to_keep,
+                grad_acc_steps=grad_acc_steps,
             )
 
         elif (
@@ -301,6 +310,7 @@ class FARMReader(BaseReader):
                 distillation_loss=distillation_loss,
                 distillation_loss_weight=distillation_loss_weight,
                 temperature=temperature,
+                grad_acc_steps=grad_acc_steps,
             )
         else:
             trainer = Trainer.create_or_load_checkpoint(
@@ -317,6 +327,7 @@ class FARMReader(BaseReader):
                 checkpoint_root_dir=Path(checkpoint_root_dir),
                 checkpoint_every=checkpoint_every,
                 checkpoints_to_keep=checkpoints_to_keep,
+                grad_acc_steps=grad_acc_steps,
             )
 
         # 5. Let it grow!
@@ -346,6 +357,7 @@ class FARMReader(BaseReader):
         checkpoints_to_keep: int = 3,
         caching: bool = False,
         cache_path: Path = Path("cache/data_silo"),
+        grad_acc_steps: int = 1,
     ):
         """
         Fine-tune a model on a QA dataset. Options:
@@ -391,6 +403,7 @@ class FARMReader(BaseReader):
         :param caching: whether or not to use caching for preprocessed dataset
         :param cache_path: Path to cache the preprocessed dataset
         :param processor: The processor to use for preprocessing. If None, the default SquadProcessor is used.
+        :param grad_acc_steps: The number of steps to accumulate gradients for before performing a backward pass.
         :return: None
         """
         return self._training_procedure(
@@ -415,6 +428,7 @@ class FARMReader(BaseReader):
             checkpoints_to_keep=checkpoints_to_keep,
             caching=caching,
             cache_path=cache_path,
+            grad_acc_steps=grad_acc_steps,
         )
 
     def distil_prediction_layer_from(
@@ -445,6 +459,7 @@ class FARMReader(BaseReader):
         distillation_loss_weight: float = 0.5,
         distillation_loss: Union[str, Callable[[torch.Tensor, torch.Tensor], torch.Tensor]] = "kl_div",
         temperature: float = 1.0,
+        grad_acc_steps: int = 1,
     ):
         """
         Fine-tune a model on a QA dataset using logit-based distillation. You need to provide a teacher model that is already finetuned on the dataset
@@ -509,6 +524,7 @@ class FARMReader(BaseReader):
         :param tinybert_learning_rate: Learning rate to use when training the student model with the TinyBERT loss function.
         :param tinybert_train_filename: Filename of training data to use when training the student model with the TinyBERT loss function. To best follow the original paper, this should be an augmented version of the training data created using the augment_squad.py script. If not specified, the training data from the original training is used.
         :param processor: The processor to use for preprocessing. If None, the default SquadProcessor is used.
+        :param grad_acc_steps: The number of steps to accumulate gradients for before performing a backward pass.
         :return: None
         """
         return self._training_procedure(
@@ -538,6 +554,7 @@ class FARMReader(BaseReader):
             distillation_loss_weight=distillation_loss_weight,
             distillation_loss=distillation_loss,
             temperature=temperature,
+            grad_acc_steps=grad_acc_steps,
         )
 
     def distil_intermediate_layers_from(
@@ -567,6 +584,7 @@ class FARMReader(BaseReader):
         distillation_loss: Union[str, Callable[[torch.Tensor, torch.Tensor], torch.Tensor]] = "mse",
         temperature: float = 1.0,
         processor: Optional[Processor] = None,
+        grad_acc_steps: int = 1,
     ):
         """
         The first stage of distillation finetuning as described in the TinyBERT paper:
@@ -623,6 +641,7 @@ class FARMReader(BaseReader):
         :param distillation_loss: Specifies how teacher and model logits should be compared. Can either be a string ("mse" for mean squared error or "kl_div" for kl divergence loss) or a callable loss function (needs to have named parameters student_logits and teacher_logits)
         :param temperature: The temperature for distillation. A higher temperature will result in less certainty of teacher outputs. A lower temperature means more certainty. A temperature of 1.0 does not change the certainty of the model.
         :param processor: The processor to use for preprocessing. If None, the default SquadProcessor is used.
+        :param grad_acc_steps: The number of steps to accumulate gradients for before performing a backward pass.
         :return: None
         """
         return self._training_procedure(
@@ -653,6 +672,7 @@ class FARMReader(BaseReader):
             temperature=temperature,
             tinybert=True,
             processor=processor,
+            grad_acc_steps=grad_acc_steps,
         )
 
     def update_parameters(
@@ -688,6 +708,58 @@ class FARMReader(BaseReader):
         self.inferencer.model.save(directory)
         self.inferencer.processor.save(directory)
 
+    def save_to_remote(
+        self, repo_id: str, private: Optional[bool] = None, commit_message: str = "Add new model to Hugging Face."
+    ):
+        """
+        Saves the Reader model to Hugging Face Model Hub with the given model_name. For this to work:
+        - Be logged in to Hugging Face on your machine via transformers-cli
+        - Have git lfs installed (https://packagecloud.io/github/git-lfs/install), you can test it by git lfs --version
+
+        :param repo_id: A namespace (user or an organization) and a repo name separated by a '/' of the model you want to save to Hugging Face
+        :param private: Set to true to make the model repository private
+        :param commit_message: Commit message while saving to Hugging Face
+        """
+        # Note: This function was inspired by the save_to_hub function in the sentence-transformers repo (https://github.com/UKPLab/sentence-transformers/)
+        # Especially for git-lfs tracking.
+
+        token = HfFolder.get_token()
+        if token is None:
+            raise ValueError(
+                "To save this reader model to Hugging Face, make sure you login to the hub on this computer by typing `transformers-cli login`."
+            )
+
+        repo_url = create_repo(token=token, repo_id=repo_id, private=private, repo_type=None, exist_ok=True)
+
+        transformer_models = self.inferencer.model.convert_to_transformers()
+
+        with tempfile.TemporaryDirectory() as tmp_dir:
+            repo = Repository(tmp_dir, clone_from=repo_url)
+
+            self.inferencer.processor.tokenizer.save_pretrained(tmp_dir)
+
+            # convert_to_transformers (above) creates one model per prediction head.
+            # As the FarmReader models only have one head (QA) we go with this.
+            transformer_models[0].save_pretrained(tmp_dir)
+
+            large_files = []
+            for root, dirs, files in os.walk(tmp_dir):
+                for filename in files:
+                    file_path = os.path.join(root, filename)
+                    rel_path = os.path.relpath(file_path, tmp_dir)
+
+                    if os.path.getsize(file_path) > (5 * 1024 * 1024):
+                        large_files.append(rel_path)
+
+            if len(large_files) > 0:
+                logger.info("Track files with git lfs: {}".format(", ".join(large_files)))
+                repo.lfs_track(large_files)
+
+            logger.info("Push model to the hub. This might take a while")
+            commit_url = repo.push_to_hub(commit_message=commit_message)
+
+        return commit_url
+
     def predict_batch(
         self,
         queries: List[str],
@@ -803,7 +875,11 @@ class FARMReader(BaseReader):
         return result
 
     def eval_on_file(
-        self, data_dir: Union[Path, str], test_filename: str, device: Optional[Union[str, torch.device]] = None
+        self,
+        data_dir: Union[Path, str],
+        test_filename: str,
+        device: Optional[Union[str, torch.device]] = None,
+        calibrate_conf_scores: bool = False,
     ):
         """
         Performs evaluation on a SQuAD-formatted file.
@@ -817,7 +893,16 @@ class FARMReader(BaseReader):
         :param device: The device on which the tensors should be processed.
                Choose from torch.device("cpu") and torch.device("cuda") (or simply "cpu" or "cuda")
                or use the Reader's device by default.
+        :param calibrate_conf_scores: Whether to calibrate the temperature for scaling of the confidence scores.
         """
+        logger.warning(
+            "FARMReader.eval_on_file() uses a slightly different evaluation approach than `Pipeline.eval()`:\n"
+            "- instead of giving you full control over which labels to use, this method always returns three types of metrics: combined (no suffix), text_answer ('_text_answer' suffix) and no_answer ('_no_answer' suffix) metrics.\n"
+            "- instead of comparing predictions with labels on a string level, this method compares them on a token-ID level. This makes it unable to do any string normalization (e.g. normalize whitespaces) beforehand.\n"
+            "Hence, results might slightly differ from those of `Pipeline.eval()`\n."
+            "If you are just about starting to evaluate your model consider using `Pipeline.eval()` instead."
+        )
+
         if device is None:
             device = self.devices[0]
         else:
@@ -840,7 +925,11 @@ class FARMReader(BaseReader):
 
         evaluator = Evaluator(data_loader=data_loader, tasks=eval_processor.tasks, device=device)
 
-        eval_results = evaluator.eval(self.inferencer.model)
+        eval_results = evaluator.eval(
+            self.inferencer.model,
+            calibrate_conf_scores=calibrate_conf_scores,
+            use_confidence_scores_for_ranking=self.use_confidence_scores,
+        )
         results = {
             "EM": eval_results[0]["EM"] * 100,
             "f1": eval_results[0]["f1"] * 100,
@@ -867,7 +956,6 @@ class FARMReader(BaseReader):
         doc_index: str = "eval_document",
         label_origin: str = "gold-label",
         calibrate_conf_scores: bool = False,
-        use_no_answer_legacy_confidence=False,
     ):
         """
         Performs evaluation on evaluation documents in the DocumentStore.
@@ -883,10 +971,16 @@ class FARMReader(BaseReader):
         :param label_index: Index/Table name where labeled questions are stored
         :param doc_index: Index/Table name where documents that are used for evaluation are stored
         :param label_origin: Field name where the gold labels are stored
-        :param calibrate_conf_scores: Whether to calibrate the temperature for temperature scaling of the confidence scores
-        :param use_no_answer_legacy_confidence: Whether to use the legacy confidence definition for no_answer: difference between the best overall answer confidence and the no_answer gap confidence.
-                                                Otherwise we use the no_answer score normalized to a range of [0,1] by an expit function (default).
+        :param calibrate_conf_scores: Whether to calibrate the temperature for scaling of the confidence scores.
         """
+        logger.warning(
+            "FARMReader.eval() uses a slightly different evaluation approach than `Pipeline.eval()`:\n"
+            "- instead of giving you full control over which labels to use, this method always returns three types of metrics: combined (no suffix), text_answer ('_text_answer' suffix) and no_answer ('_no_answer' suffix) metrics.\n"
+            "- instead of comparing predictions with labels on a string level, this method compares them on a token-ID level. This makes it unable to do any string normalization (e.g. normalize whitespaces) beforehand.\n"
+            "Hence, results might slightly differ from those of `Pipeline.eval()`\n."
+            "If you are just about starting to evaluate your model consider using `Pipeline.eval()` instead."
+        )
+
         if device is None:
             device = self.devices[0]
         else:
@@ -996,7 +1090,6 @@ class FARMReader(BaseReader):
             self.inferencer.model,
             calibrate_conf_scores=calibrate_conf_scores,
             use_confidence_scores_for_ranking=self.use_confidence_scores,
-            use_no_answer_legacy_confidence=use_no_answer_legacy_confidence,
         )
         toc = perf_counter()
         reader_time = toc - tic
