@@ -2,13 +2,15 @@ import itertools
 from typing import List, Optional, Set, Union
 
 import logging
+
+from tqdm.auto import tqdm
 from transformers import pipeline
 from transformers.models.auto.modeling_auto import AutoModelForSeq2SeqLM
 
 from haystack.schema import Document
 from haystack.nodes.summarizer.base import BaseSummarizer
 from haystack.modeling.utils import initialize_device_settings
-
+from haystack.utils.torch_utils import ListDataset
 
 logger = logging.getLogger(__name__)
 
@@ -61,7 +63,8 @@ class TransformersSummarizer(BaseSummarizer):
         clean_up_tokenization_spaces: bool = True,
         separator_for_single_summary: str = " ",
         generate_single_summary: bool = False,
-        batch_size: Optional[int] = None,
+        batch_size: int = 16,
+        progress_bar: bool = True,
     ):
         """
         Load a Summarization model from Transformers.
@@ -84,6 +87,7 @@ class TransformersSummarizer(BaseSummarizer):
                                         be summarized.
                                         Important: The summary will depend on the order of the supplied documents!
         :param batch_size: Number of documents to process at a time.
+        :param progress_bar: Whether to show a progress bar.
         """
         super().__init__()
 
@@ -103,6 +107,7 @@ class TransformersSummarizer(BaseSummarizer):
         self.generate_single_summary = generate_single_summary
         self.print_log: Set[str] = set()
         self.batch_size = batch_size
+        self.progress_bar = progress_bar
 
     def predict(self, documents: List[Document], generate_single_summary: Optional[bool] = None) -> List[Document]:
         """
@@ -158,9 +163,15 @@ class TransformersSummarizer(BaseSummarizer):
 
         result: List[Document] = []
 
-        for context, summarized_answer in zip(contexts, summaries):
-            cur_doc = Document(content=summarized_answer["summary_text"], meta={"context": context})
-            result.append(cur_doc)
+        if generate_single_summary:
+            for context, summarized_answer in zip(contexts, summaries):
+                cur_doc = Document(content=summarized_answer["summary_text"], meta={"context": context})
+                result.append(cur_doc)
+        else:
+            for context, summarized_answer, document in zip(contexts, summaries, documents):
+                cur_doc = Document(content=summarized_answer["summary_text"], meta=document.meta)
+                cur_doc.meta.update({"context": context})
+                result.append(cur_doc)
 
         return result
 
@@ -237,15 +248,24 @@ class TransformersSummarizer(BaseSummarizer):
                 logger.warning(truncation_warning)
                 break
 
-        summaries = self.summarizer(
-            contexts,
-            min_length=self.min_length,
-            max_length=self.max_length,
-            return_text=True,
-            clean_up_tokenization_spaces=self.clean_up_tokenization_spaces,
-            truncation=True,
-            batch_size=batch_size,
-        )
+        summaries = []
+        # HF pipeline progress bar hack, see https://discuss.huggingface.co/t/progress-bar-for-hf-pipelines/20498/2
+        summaries_dataset = ListDataset(contexts)
+        for summary_batch in tqdm(
+            self.summarizer(
+                summaries_dataset,
+                min_length=self.min_length,
+                max_length=self.max_length,
+                return_text=True,
+                clean_up_tokenization_spaces=self.clean_up_tokenization_spaces,
+                truncation=True,
+                batch_size=batch_size,
+            ),
+            disable=not self.progress_bar,
+            total=len(summaries_dataset),
+            desc="Summarizing",
+        ):
+            summaries.extend(summary_batch)
 
         # Group summaries together
         grouped_summaries = []
