@@ -231,6 +231,8 @@ class BaseStandardPipeline(ABC):
             "document_id_or_answer",
         ] = "document_id_or_answer",
         answer_scope: Literal["any", "context", "document_id", "document_id_and_context"] = "any",
+        wrong_examples_fields: List[str] = ["answer", "context", "document_id"],
+        max_characters_per_field: int = 150,
     ):
         """
         Prints evaluation report containing a metrics funnel and worst queries for further analysis.
@@ -265,6 +267,8 @@ class BaseStandardPipeline(ABC):
             - 'document_id_and_context': The answer is only considered correct if its document ID and its context match as well.
             The default value is 'any'.
             In Question Answering, to enforce that the retrieved document is considered correct whenever the answer is correct, set `document_scope` to 'answer' or 'document_id_or_answer'.
+        :param wrong_examples_fields: A list of field names to include in the worst samples.
+        :param max_characters_per_field: The maximum number of characters per wrong example to show (per field).
         """
         if metrics_filter is None:
             metrics_filter = self.metrics_filter
@@ -274,6 +278,8 @@ class BaseStandardPipeline(ABC):
             metrics_filter=metrics_filter,
             document_scope=document_scope,
             answer_scope=answer_scope,
+            wrong_examples_fields=wrong_examples_fields,
+            max_characters_per_field=max_characters_per_field,
         )
 
     def run_batch(self, queries: List[str], params: Optional[dict] = None, debug: Optional[bool] = None):
@@ -517,6 +523,11 @@ class TranslationWrapperPipeline(BaseStandardPipeline):
 
         self.pipeline = Pipeline()
         self.pipeline.add_node(component=input_translator, name="InputTranslator", inputs=["Query"])
+        # Make use of run_batch instead of run for output_translator if pipeline is a QuestionAnswerGenerationPipeline,
+        # as the reader's run method is overwritten by its run_batch method, which is incompatible with the translator's
+        # run method.
+        if isinstance(pipeline, QuestionAnswerGenerationPipeline):
+            setattr(output_translator, "run", output_translator.run_batch)
 
         graph = pipeline.pipeline.graph
         previous_node_name = ["InputTranslator"]
