@@ -1,5 +1,6 @@
 import logging
 
+
 def cache_models():
     """
     Small function that caches models and other data.
@@ -8,11 +9,13 @@ def cache_models():
     # download punkt tokenizer
     logging.info("Caching punkt data")
     import nltk
-    nltk.download('punkt', download_dir='/root/nltk_data')
-    
+
+    nltk.download("punkt", download_dir="/root/nltk_data")
+
     # Cache roberta-base-squad2 model
     logging.info("Caching deepset/roberta-base-squad2")
     import transformers
-    model_to_cache='deepset/roberta-base-squad2'
+
+    model_to_cache = "deepset/roberta-base-squad2"
     transformers.AutoTokenizer.from_pretrained(model_to_cache)
-    transformers.AutoModel.from_pretrained(model_to_cache)
\ No newline at end of file
+    transformers.AutoModel.from_pretrained(model_to_cache)
